/**
 * Event manager interface
 */
export interface IEventManager {
    subscribeToLocalEvent(eventType: string, handler: Function): any;
    unsubscribeFromLocalEvent(eventType: string, handler?: Function): any;
    isSubscribedToLocalEvent(eventType: string): boolean;
    publishLocalEvent(eventType: string, data: any): any;
}
/**
 * Local storage interface
 */
export interface ILocalStorageData {
    getString(key: string): Promise<string>;
    setString(key: string, data: string): Promise<boolean>;
    getObject(key: string): Promise<Object>;
    setObject(key: string, data: Object): Promise<boolean>;
    remove(key: string): Promise<boolean>;
}
/**
 * Log level enum
 */
export declare enum LogLevels {
    None = 0,
    Error = 1,
    Warn = 2,
    Debug = 3
}
/**
 * Log persistence enum
 */
export declare enum LogPersistences {
    None = 0,
    IndexedDB = 1,
    LocalStorage = 2
}
/**
 * Log persistence enum
 */
export declare enum OrphanedEventPersistences {
    None = 0,
    IndexedDbIfSupported = 1,
    LocalStorage = 2
}
/**
 * Log Event interface
 */
export interface ILogEvent {
    key?: number;
    logLevel: LogLevels;
    message: string;
    created: number;
    timestamp: string;
    data: any;
}
/**
 * Logger interface
 */
export interface ILogger {
    logLevel: LogLevels;
    log(message: String, data?: Object): Promise<boolean>;
    warn(message: String, data?: Object): Promise<boolean>;
    error(message: String, data?: Object): Promise<boolean>;
    clearLog(): Promise<boolean>;
    getLog(): Promise<string>;
}
/**
 * Rest client result interface
 */
export interface IRestClientResult {
    statusCode: number;
    response: any;
    headers: any;
}
/**
 * Rest client interface
 */
export interface IRestClient {
    get(url: string, headers?: any): Promise<IRestClientResult>;
    post(url: string, headers: any, data: any): Promise<IRestClientResult>;
    put(url: string, headers: any, data: any): Promise<IRestClientResult>;
    patch(url: string, headers: any, data: any): Promise<IRestClientResult>;
    delete(url: string, headers: any): Promise<IRestClientResult>;
}
/**
 *
 */
export interface IApiSpaceInfo {
    createdOn: string;
    id: string;
    name: string;
    updatedOn: string;
}
/**
 *
 */
export interface IApiSpaceAuthInfo {
    audience: string;
    idClaim: string;
    issuer: string;
    sharedSecret: string;
}
/**
 *
 */
export interface IApiSpaceManager {
    getToken(accountId: string, profileId: string): Promise<any>;
    createApiSpace(token: string, name: string): Promise<IApiSpaceInfo>;
    updateAuth(token: string, apiSpaceId: string, authInfo: IApiSpaceAuthInfo): Promise<IApiSpaceAuthInfo>;
}
/**
 *  returned from /appspaces/{appSpaceId}/sessions/start
 *  GET
 *  {
 *      "authenticationId": "string",
 *      "provider": "jsonWebToken",
 *      "nonce": "string",
 *      "expiresOn": "2016-04-07T11:17:10.935Z"
 *  }
 */
export interface ISessionStartResponse {
    /**
     * @proprty (string) authenticationId - authenticationId generated by auth service
     */
    authenticationId: string;
    /**
     * @proprty (string) provider - "jsonWebToken"
     */
    provider: string;
    /**
     * @proprty (string) nonce - nonce generated by auth service (controlled in app space settings)
     */
    nonce: string;
    /**
     * @proprty (string) expiresOn - ISO date string
     */
    expiresOn: string;
}
/**
 *
 */
export interface ISession {
    id: string;
    profileId: string;
    deviceId: string;
    platform: string;
    platformVersion: string;
    sdkType: string;
    sdkVersion: string;
    sourceIp: string;
    isActive: boolean;
    createdOn: string;
    expiresOn: string;
}
/**
 *
 */
export interface ISessionInfo {
    token: string;
    session: ISession;
}
/**
 *
 */
export interface IBrowserInfo {
    name: string;
    version: string;
}
/**
 * Environment enum
 */
export declare enum Environment {
    development = 0,
    production = 1
}
/**
 * Environment enum
 */
export interface IPushConfig {
    fcm?: {
        package: string;
        registrationId: string;
    };
    apns?: {
        bundleId: string;
        environment: Environment;
        token: string;
    };
}
/**
 * Session manager interface
 */
export interface ISessionManager {
    sessionInfo: ISessionInfo;
    initialise(): Promise<boolean>;
    getValidToken(): Promise<string>;
    startSession(): Promise<ISessionInfo>;
    endSession(): Promise<boolean>;
    requestSession(): Promise<any>;
    removeSession(): Promise<boolean>;
}
/**
 * Network manager interface
 */
export interface INetworkManager {
    session: ISession;
    getValidToken(): Promise<string>;
    startSession(): Promise<ISessionInfo>;
    restartSession(): Promise<ISessionInfo>;
    endSession(): Promise<boolean>;
    ensureSession(): Promise<ISessionInfo>;
    setWebsocketEnabled(enable: boolean): Promise<boolean>;
}
/**
 * Device manager interface
 */
export interface IDeviceManager {
    setFCMPushDetails(sessionId: string, packageName: string, registrationId: string): Promise<boolean>;
    setAPNSPushDetails(sessionId: string, bundleId: string, environment: Environment, token: string): Promise<boolean>;
    removePushDetails(sessionId: string): Promise<boolean>;
}
/**
 * Facebook manager interface
 */
export interface IFacebookManager {
    createSendToMessengerState(data?: any): Promise<any>;
}
/**
 * Auth Challenge options definition
 */
export interface IAuthChallengeOptions {
    nonce: string;
}
/**
 * Auth Challenge callback signature
 */
export interface IAuthChallenge {
    (options: IAuthChallengeOptions, answerAuthenticationChallenge: Function): void;
}
export interface IFoundationRestUrls {
    content: string;
    conversations: string;
    conversation: string;
    participants: string;
    typing: string;
    push: string;
    facebook: string;
    events: string;
    messages: string;
    statusUpdates: string;
    profiles: string;
    profile: string;
    sessions: string;
    sessionStart: string;
    session: string;
}
export interface IEventMapping {
    [category: string]: string[];
}
/**
 * Comapi options to be passed in on startup
 */
export interface IComapiConfig {
    apiSpaceId: string;
    logRetentionHours: number;
    authChallenge?: IAuthChallenge;
    urlBase?: string;
    webSocketBase?: string;
    logLevel?: LogLevels;
    logPersistence?: LogPersistences;
    isTypingTimeout?: number;
    isTypingOffTimeout?: number;
    foundationRestUrls?: IFoundationRestUrls;
    eventMapping?: IEventMapping;
    interfaceContainer?: any;
    localStoragePrefix?: string;
    orphanedEventPersistence?: OrphanedEventPersistences;
    enableWebsocketForNonChatUsage?: boolean;
    pushConfig?: IPushConfig;
}
export interface IContentData {
    file: File;
    data: string;
    name: string;
    type: string;
}
export interface IUploadContentResult {
    folder: string;
    id: string;
    type: string;
    url: string;
    size: number;
    name: string;
}
export interface IContentManager {
    uploadContent(content: IContentData, folder?: string): Promise<IUploadContentResult>;
}
/**
 * Profile manager interface
 */
export interface IProfileManager {
    getProfile(id: string): Promise<any>;
    updateProfile(id: string, profile: Object, eTag?: string): Promise<any>;
    patchProfile(id: string, profile: Object, eTag?: string): Promise<any>;
    queryProfiles(query: string): Promise<any>;
}
/**
 * Log level enum
 */
export declare enum ConversationScope {
    public = 0,
    participant = 1
}
/**
 * Conversation manager interface
 */
export interface IConversationManager {
    createConversation(conversationDetails: IConversationDetails): Promise<IConversationDetails2>;
    updateConversation(conversationDetails: IConversationDetails, eTag?: string): Promise<IConversationDetails2>;
    deleteConversation(conversationId: string): Promise<boolean>;
    getConversation(conversationId: string): Promise<IConversationDetails2>;
    addParticipantsToConversation(conversationlId: string, participants: IConversationParticipant[]): Promise<boolean>;
    deleteParticipantsFromConversation(conversationId: string, participants: string[]): Promise<boolean>;
    getParticipantsInConversation(conversationId: string): Promise<IConversationParticipant[]>;
    getConversations(scope?: ConversationScope, profileId?: string): Promise<IConversationDetails2[]>;
    sendIsTyping(conversationId: string): Promise<boolean>;
    sendIsTypingOff(conversationId: string): Promise<boolean>;
}
/**
 *
 */
export interface IConversationPrivelages {
    canSend: boolean;
    canAddParticipants: boolean;
    canRemoveParticipants: boolean;
}
/**
 *
 */
export interface IConversationRoles {
    owner: IConversationPrivelages;
    participant: IConversationPrivelages;
}
/**
 *
 */
export interface IConversationParticipant {
    id: string;
    role?: string;
}
/**
 * Conversation details
 */
export interface IConversationDetails {
    id: string;
    name: string;
    description?: string;
    roles: IConversationRoles;
    isPublic: boolean;
}
/**
 * Conversation details interface
 */
export interface IConversationDetails2 extends IConversationDetails {
    _createdOn: string;
    _etag?: string;
    _updatedOn: string;
    latestSentEventId?: number;
    participantCount?: number;
}
export interface ISendMessageResult {
    id: string;
    eventId: number;
}
/**
 *
 */
export interface IConversationMessage {
    id?: string;
    sentEventId?: number;
    metadata?: any;
    context?: any;
    /**
     *   "context": {
     *       "from": {
     *           "id": "stevanl"
     *       },
     *       "conversationId": "bc24d5b0-b03c-4594-872b-510c4af81dfe",
     *       "sentBy": "stevanl",
     *       "sentOn": "2016-10-19T11:51:26.349Z"
     *   },
     */
    parts?: IMessagePart[];
    alert?: IMessageAlert;
    statusUpdates?: any;
}
/**
 *
 */
export interface IConversationMessagesResult {
    earliestEventId: number;
    latestEventId: number;
    messages: IConversationMessage[];
    orphanedEvents?: any[];
}
/**
 * Message manager interface
 */
export interface IMessageManager {
    sendMessageToConversation(conversationId: string, message: IConversationMessage): Promise<ISendMessageResult>;
    getConversationMessages(conversation: string, limit: number, from?: number): Promise<IConversationMessagesResult>;
    sendMessageStatusUpdates(conversation: string, statuses: IMessageStatus[]): Promise<any>;
    getConversationEvents(conversation: string, from: number, limit: number): Promise<IConversationMessageEvent[]>;
}
/**
 *
 */
export interface IMessagePart {
    /**
     * The message part name
     */
    name?: string;
    /**
     * The messahe Part Type
     */
    type?: string;
    /**
     * The message URL
     */
    url?: string;
    /**
     * Te message data
     */
    data?: string;
    /**
     * The size of the data
     */
    size?: number;
}
/**
 *
 */
export interface IApnsAlert {
    badge?: number;
    sound?: string;
    alert: string;
    payload?: any;
}
/**
 *
 */
export interface IFcmAlert {
    collapse_key?: string;
    data?: any;
    notification: {
        title: string;
        body: string;
    };
}
/**
 *
 */
export interface IMessageAlert {
    text: string;
    platforms: {
        apns: IApnsAlert;
        fcm: IFcmAlert;
    };
}
/**
 *
 */
export interface IMessageStatus {
    messageIds: string[];
    status: string;
    timestamp: string;
}
export interface IEventMapper {
    conversationDeleted(event: any): IConversationDeletedEventData;
    conversationUndeleted(event: any): IConversationUndeletedEventData;
    conversationUpdated(event: any): IConversationUpdatedEventData;
    participantAdded(event: any): IParticipantAddedEventData;
    participantRemoved(event: any): IParticipantRemovedEventData;
    participantTyping(event: any): IParticipantTypingEventData;
    participantTypingOff(event: any): IParticipantTypingOffEventData;
    conversationMessageSent(event: any): IConversationMessageEvent;
    conversationMessageRead(event: any): IConversationMessageEvent;
    conversationMessageDelivered(event: any): IConversationMessageEvent;
    profileUpdated(event: any): IProfileUpdatedEvent;
}
/**
 *
 */
export interface IWebSocketManager {
    readonly isEnabled: boolean;
    connect(): Promise<boolean>;
    disconnect(): Promise<boolean>;
    isConnected(): boolean;
    hasSocket(): boolean;
    send(data: any): void;
    generateInterval(k: number): number;
    setWebsocketEnabled(enable: boolean): Promise<boolean>;
}
export interface IMessageSentPayload {
    messageId: string;
    metadata: any;
    context: any;
    parts: IMessagePart[];
    alert: IMessageAlert;
}
export interface IMessageStatusUpdatePayload {
    messageId: string;
    conversationId: string;
    profileId: string;
    timestamp: string;
}
export interface IConversationMessageEvent {
    eventId: string;
    name: string;
    conversationId: string;
    conversationEventId: number;
    payload: IMessageSentPayload | IMessageStatusUpdatePayload;
}
/**
 * TODO: do we want a profileId in here ?
 */
export interface IConversationDeletedEventData {
    conversationId: string;
    createdBy: string;
    timestamp: string;
}
/**
 * TODO: do we want a profileId in here ?
 */
export interface IConversationUndeletedEventData {
    conversationId: string;
    createdBy: string;
    timestamp: string;
}
/**
 *
 */
export interface IConversationUpdatedEventData {
    conversationId: string;
    createdBy: string;
    name: string;
    description: string;
    roles: IConversationRoles;
    isPublic: boolean;
    timestamp: string;
    eTag: string;
}
/**
 *
 */
export interface IParticipantAddedEventData {
    conversationId: string;
    createdBy: string;
    profileId: string;
    role: string;
    timestamp: string;
}
/**
 *
 */
export interface IParticipantRemovedEventData {
    conversationId: string;
    createdBy: string;
    profileId: string;
    timestamp: string;
}
/**
 *
 */
export interface IParticipantTypingEventData {
    conversationId: string;
    createdBy: string;
    profileId: string;
    timestamp: string;
}
/**
 *
 */
export interface IParticipantTypingOffEventData {
    conversationId: string;
    createdBy: string;
    profileId: string;
    timestamp: string;
}
/**
 *
 */
export interface IProfileUpdatedEvent {
    eTag: string;
    profile: any;
}
/**
 *
 */
export interface IGetMessagesResponse {
    continuationToken?: number;
    earliestEventId?: number;
    latestEventId?: number;
    messages: IConversationMessage[];
}
export interface IAppMessaging {
    createConversation(conversationDetails: IConversationDetails): Promise<IConversationDetails2>;
    updateConversation(conversationDetails: IConversationDetails, eTag?: string): Promise<IConversationDetails2>;
    getConversation(conversationId: string): Promise<IConversationDetails2>;
    deleteConversation(conversationId: string): Promise<boolean>;
    addParticipantsToConversation(conversationId: string, participants: IConversationParticipant[]): Promise<boolean>;
    deleteParticipantsFromConversation(conversationId: string, participants: string[]): Promise<boolean>;
    getParticipantsInConversation(conversationId: string): Promise<IConversationParticipant[]>;
    getConversations(scope?: ConversationScope, profileId?: string): Promise<IConversationDetails2[]>;
    getConversationEvents(conversationId: string, from: number, limit: number): Promise<IConversationMessageEvent[]>;
    sendMessageToConversation(conversationId: string, message: IConversationMessage): Promise<ISendMessageResult>;
    sendMessageStatusUpdates(conversationId: string, statuses: IMessageStatus[]): Promise<any>;
    getMessages(conversationId: string, pageSize: number, continuationToken?: number): Promise<IGetMessagesResponse>;
    sendIsTyping(conversationId: string): Promise<boolean>;
    sendIsTypingOff(conversationId: string): Promise<boolean>;
    uploadContent(content: IContentData, folder?: string): Promise<IUploadContentResult>;
}
export interface IAppMessagingInternal extends IAppMessaging {
    enableSocket(): void;
}
export interface IProfile {
    getProfile(profileId: string): Promise<any>;
    queryProfiles(query?: string): Promise<any>;
    updateProfile(profileId: string, profile: any, eTag?: string): Promise<any>;
    patchProfile(id: string, profile: Object, eTag?: string): Promise<any>;
    getMyProfile(useEtag?: boolean): Promise<any>;
    updateMyProfile(profile: any, useEtag?: boolean): Promise<any>;
    patchMyProfile(profile: any, useEtag?: boolean): Promise<any>;
}
export interface IServices {
    appMessaging: IAppMessaging;
    profile: IProfile;
}
export interface IDevice {
    setFCMPushDetails(packageName: string, registrationId: string): Promise<boolean>;
    setAPNSPushDetails(bundleId: string, environment: Environment, token: string): Promise<boolean>;
    removePushDetails(): Promise<boolean>;
}
export interface IChannels {
    createFbOptInState(data?: any): Promise<any>;
}
export interface IOrphanedEventManager {
    clearAll(): Promise<boolean>;
    clear(conversationId: string): Promise<boolean>;
    getContinuationToken(conversationId: string): Promise<number>;
    setContinuationToken(conversationId: string, continuationToken: number): Promise<boolean>;
    addOrphanedEvent(event: IConversationMessageEvent): Promise<boolean>;
    removeOrphanedEvent(event: IConversationMessageEvent): Promise<boolean>;
    getOrphanedEvents(conversationId: string): Promise<IConversationMessageEvent[]>;
}
export interface IMessagePager {
    getMessages(conversationId: string, pageSize: number, continuationToken?: number): Promise<IGetMessagesResponse>;
    getOrphanedEvents(conversationId: string, orphanedEvents: any[]): Promise<IConversationMessageEvent[]>;
    markMessagesAsDelivered(id: string, messages: IConversationMessage[], userId: string): Promise<string>;
    resetConversation(conversationId: string): Promise<boolean>;
}
/**
 * Foundation interface definition
 * static methods missing as cant define them in TS ;-(
 */
export interface IFoundation {
    services: IServices;
    device: IDevice;
    channels: IChannels;
    session: ISession;
    logger: ILogger;
    startSession(): Promise<ISession>;
    endSession(): Promise<boolean>;
    on(eventType: string, handler: Function): void;
    off(eventType: string, handler?: Function): void;
    getLogs(): Promise<string>;
}
/**
 * Method to perform the asnc operation
 * @param {any} data  the data to operate on
 * @returns {Promise<any>} returns a promise
 */
export declare type DoUntilOperationFunction = (data: any) => Promise<any>;
/**
 * Method to decide whether to continue or not
 * @param {any} data  the data to look at (will have been returned vi a promise from DoUntilOperationFunction)
 * @returns {boolean} returns true or false. return false to stop
 */
export declare type DoUntilTestFunction = (data: any) => boolean;
